# Xfers Python bindings
# API docs at http://docs.xfers.io/

# Configuration variables

api_key = None
api_base = ''

SG_SANDBOX_BASE = 'https://sandbox.xfers.io/api/v3'
SG_PRODUCTION_BASE = 'https://www.xfers.io/api/v3'
ID_SANDBOX_BASE = 'https://sandbox-id.xfers.com/api/v3'
ID_PRODUCTION_BASE = 'https://id.xfers.com/api/v3'


def set_sg_sandbox():
    global api_base
    api_base = SG_SANDBOX_BASE


def set_sg_production():
    global api_base
    api_base = SG_PRODUCTION_BASE


def set_id_sandbox():
    global api_base
    api_base = ID_SANDBOX_BASE


def set_id_production():
    global api_base
    api_base = ID_PRODUCTION_BASE



import requests
import xfers
import json
from error import (XfersError, InvalidRequestError, AuthenticationError, InternalServerError)


def create_auth_headers(connect_key):
    if connect_key:
        return {'X-XFERS-APP-API-KEY': connect_key}
    else:
        return {'X-XFERS-USER-API-KEY': xfers.api_key}


def handle_api_error(msg, status_code):
    if status_code == 400:
        raise InvalidRequestError(msg, status_code)
    elif status_code == 401:
        raise AuthenticationError(msg, status_code)
    elif status_code == 500:
        raise InternalServerError(msg, status_code)
    else:
        raise XfersError(msg, status_code)


def get(params, resource_url, connect_key):
    url = xfers.api_base + resource_url
    r = requests.get(url, headers=create_auth_headers(connect_key), params=params)
    if r.status_code != 200:
        handle_api_error(r.text, r.status_code)
    return json.loads(r.text)


def post(params, resource_url, connect_key):
    url = xfers.api_base + resource_url
    r = requests.post(url, headers=create_auth_headers(connect_key), data=params)
    if r.status_code != 200:
        handle_api_error(r.text, r.status_code)
    return json.loads(r.text)


def put(params, resource_url, connect_key):
    url = xfers.api_base + resource_url
    r = requests.put(url, headers=create_auth_headers(connect_key), data=params)
    if r.status_code != 200:
        handle_api_error(r.text, r.status_code)
    return json.loads(r.text)


def delete(resource_url, connect_key):
    url = xfers.api_base + resource_url
    r = requests.delete(url, headers=create_auth_headers(connect_key))
    if r.status_code != 200:
        handle_api_error(r.text, r.status_code)
    return json.loads(r.text)




from xfers import api_resource
from xfers import error


RESOURCE_URL = '/payouts'


def retrieve(payout_id, connect_key=None):
    if not payout_id:
        raise error.InvalidRequestError("Payout id cannot be empty", 400)
    return api_resource.get(None, '{}/{}'.format(RESOURCE_URL, payout_id), connect_key)


def list_all(params=None, connect_key=None):
    return api_resource.get(params, RESOURCE_URL, connect_key)


def create(params, connect_key=None):
    if not params:
        raise error.InvalidRequestError("Params cannot be empty", 400)
    return api_resource.post(params, RESOURCE_URL, connect_key)



from xfers import api_resource
from xfers import error


RESOURCE_URL = '/authorize'


def authorize(params, connect_key=None):
    if not params:
        raise error.InvalidRequestError("Params cannot be empty", 400)
    return api_resource.post(params, '{}/signup_login'.format(RESOURCE_URL), connect_key)


def get_token(params, connect_key=None):
    if not params:
        raise error.InvalidRequestError("Params cannot be empty", 400)
    return api_resource.get(params, '{}/get_token'.format(RESOURCE_URL), connect_key)



from xfers import api_resource
from xfers import error


RESOURCE_URL = '/user'


def retrieve(connect_key=None):
    return api_resource.get(None, RESOURCE_URL, connect_key)


def update(params=None, connect_key=None):
    if not params:
        raise error.InvalidRequestError("Params cannot be empty", 400)
    return api_resource.put(params, RESOURCE_URL, connect_key)


def transfer_info(connect_key=None):
    return api_resource.get(None, '{}/transfer_info'.format(RESOURCE_URL), connect_key)


def activities(connect_key=None):
    return api_resource.get(None, '{}/activities'.format(RESOURCE_URL), connect_key)['activities']



from xfers import api_resource
from xfers import error


RESOURCE_URL = '/user/bank_account'


def list_all(connect_key=None):
    return api_resource.get(None, RESOURCE_URL, connect_key)


def add(params, connect_key=None):
    if not params:
        raise error.InvalidRequestError("Params cannot be empty", 400)
    return api_resource.post(params, RESOURCE_URL, connect_key)


def update(bank_account_id, params, connect_key=None):
    if not bank_account_id or not params:
        raise error.InvalidRequestError("Bank account id and params cannot be empty", 400)
    return api_resource.put(params, '{}/{}'.format(RESOURCE_URL, bank_account_id), connect_key)


def delete(bank_account_id, connect_key=None):
    if not bank_account_id:
        raise error.InvalidRequestError("Bank account id cannot be empty", 400)
    return api_resource.delete('{}/{}'.format(RESOURCE_URL, bank_account_id), connect_key)


def withdraw(bank_account_id, params, connect_key=None):
    if not bank_account_id or not params:
        raise error.InvalidRequestError("Bank account id and params cannot be empty", 400)
    return api_resource.post(params, '{}/{}/withdraw'.format(RESOURCE_URL, bank_account_id), connect_key)


def withdrawal_requests(params=None, connect_key=None):
    return api_resource.get(params, '{}/withdrawal_requests'.format(RESOURCE_URL), connect_key)['withdrawal_requests']



from xfers import api_resource
from xfers import error


RESOURCE_URL = '/charges'


def retrieve(charge_id, connect_key=None):
    if not charge_id:
        raise error.InvalidRequestError("Charge id cannot be empty", 400)
    return api_resource.get(None, '{}/{}'.format(RESOURCE_URL, charge_id), connect_key)


def list_all(params=None, connect_key=None):
    return api_resource.get(params, RESOURCE_URL, connect_key)


def create(params, connect_key=None):
    if not params:
        raise error.InvalidRequestError("Params cannot be empty", 400)
    return api_resource.post(params, RESOURCE_URL, connect_key)


def refund(charge_id, connect_key=None):
    if not charge_id:
        raise error.InvalidRequestError("Charge id cannot be empty", 400)
    return api_resource.post(None, '{}/{}/refunds'.format(RESOURCE_URL, charge_id), connect_key)


def validate(charge_id, params, connect_key=None):
    if not charge_id:
        raise error.InvalidRequestError("Charge id cannot be empty", 400)
    return api_resource.post(params, '{}/{}/validate'.format(RESOURCE_URL, charge_id), connect_key)


def cancel(charge_id, connect_key=None):
    if not charge_id:
        raise error.InvalidRequestError("Charge id cannot be empty", 400)
    return api_resource.post(None, '{}/{}/cancel'.format(RESOURCE_URL, charge_id), connect_key)


def settle(charge_id, settlement_code=None, connect_key=None):
    if not charge_id:
        raise error.InvalidRequestError("Charge id cannot be empty", 400)
    params = None
    if settlement_code:
        params = {'settlement_code': settlement_code}
    return api_resource.post(params, '{}/{}'.format(RESOURCE_URL, charge_id), connect_key)
